from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class HtmxMessagesAppConfig(AppConfig):
    """App config for Django Htmx Messages."""

    name = "django_htmx_messages"
    verbose_name = _("htmx messages")



from django.contrib.messages import constants as message_constants
from django.contrib.messages.storage.base import Message
from django.http import HttpResponse
from django.test import RequestFactory, TestCase

from django_htmx_messages.middleware import HtmxMessageMiddleware


class TestHtmxMessageMiddleware(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

        def get_response(request):
            return HttpResponse()

        self.middleware = HtmxMessageMiddleware(get_response=get_response)

    def test_non_htmx_request_returns_unmodified(self):
        """Test that non-HTMX requests are returned unmodified."""
        request = self.factory.get("/")
        response = HttpResponse("Test response")

        processed_response = self.middleware.process_response(request, response)

        self.assertEqual(processed_response.content, b"Test response")

    def test_redirect_response_returns_unmodified(self):
        """Test that redirect responses are returned unmodified."""
        request = self.factory.get("/", HTTP_HX_REQUEST="true")
        response = HttpResponse(status=302)

        processed_response = self.middleware.process_response(request, response)

        self.assertEqual(processed_response.status_code, 302)
        self.assertEqual(processed_response.content, b"")

    def test_htmx_redirect_header_returns_unmodified(self):
        """Test that responses with HX-Redirect header are returned unmodified."""
        request = self.factory.get("/", HTTP_HX_REQUEST="true")
        response = HttpResponse()
        response.headers["HX-Redirect"] = "/new-location/"

        processed_response = self.middleware.process_response(request, response)

        self.assertEqual(processed_response.content, b"")

    def test_htmx_request_without_messages_returns_unmodified(self):
        """Test that HTMX requests without messages are returned unmodified."""
        request = self.factory.get("/", HTTP_HX_REQUEST="true")
        response = HttpResponse("Test response")

        processed_response = self.middleware.process_response(request, response)

        self.assertEqual(processed_response.content, b"Test response")

    def test_htmx_request_with_messages_adds_toast(self):
        """Test that HTMX requests with messages have toast content appended."""
        request = self.factory.get("/", HTTP_HX_REQUEST="true")
        request._messages = []
        request._messages.append(
            Message(level=message_constants.SUCCESS, message="Test message")
        )
        response = HttpResponse("<div>Original content</div>")

        processed_response = self.middleware.process_response(request, response)

        # Check that original content is preserved
        self.assertIn(b"<div>Original content</div>", processed_response.content)
        # Check that toast container is added with correct classes
        self.assertIn(
            b'class="toast-container position-fixed top-0 end-0 p-3"',
            processed_response.content,
        )
        # Check that message is included
        self.assertIn(b"Test message", processed_response.content)

    def test_multiple_messages_all_included(self):
        """Test that multiple messages are all included in the response."""
        request = self.factory.get("/", HTTP_HX_REQUEST="true")
        request._messages = []
        messages = [
            Message(level=message_constants.SUCCESS, message="Success message"),
            Message(level=message_constants.ERROR, message="Error message"),
            Message(level=message_constants.INFO, message="Info message"),
        ]
        for msg in messages:
            request._messages.append(msg)
        response = HttpResponse()

        processed_response = self.middleware.process_response(request, response)

        for msg in messages:
            self.assertIn(msg.message.encode(), processed_response.content)

    def test_message_levels_included_as_classes(self):
        """Test that message levels are included as CSS classes."""
        request = self.factory.get("/", HTTP_HX_REQUEST="true")
        request._messages = []
        request._messages.append(
            Message(level=message_constants.SUCCESS, message="Success message")
        )
        response = HttpResponse()

        processed_response = self.middleware.process_response(request, response)

        # Check for the Bootstrap-styled success class
        self.assertIn(
            b'class="toast align-items-center border-0 text-white bg-success"',
            processed_response.content,
        )

    def test_middleware_chain(self):
        """Test that the middleware properly chains the get_response call."""
        request = self.factory.get("/")
        response = self.middleware(request)
        self.assertIsInstance(response, HttpResponse)



from django.contrib.messages import get_messages
from django.http import HttpRequest, HttpResponse
from django.template.loader import render_to_string
from django.utils.deprecation import MiddlewareMixin


class HtmxMessageMiddleware(MiddlewareMixin):
    """
    Middleware that handles Django messages in HTMX responses.

    Middleware that moves messages into the HX-Trigger
    header when request is made with HTMX.
    """

    def process_response(
        self, request: HttpRequest, response: HttpResponse
    ) -> HttpResponse:
        """
        Process HTMX responses by injecting Django messages as toast notifications.

        Args:
            request: The HTTP request
            response: The HTTP response

        Returns:
            The modified response with injected messages

        """
        # The HX-Request header indicates that the request was made with HTMX
        if "HX-Request" not in request.headers:
            return response

        # Ignore HTTP redirections because HTMX cannot read the body
        if 300 <= response.status_code < 400:
            return response

        # Ignore client-side redirection because HTMX drops OOB swaps
        if "HX-Redirect" in response.headers:
            return response

        # Extract the messages
        messages = get_messages(request)
        if not messages:
            return response

        response.write(
            render_to_string(
                template_name="toasts.html",
                context={"messages": messages},
                request=request,
            )
        )

        return response



from .conf import app_settings


def add(n1: int, n2: int) -> int:
    """Add the arguments."""
    return n1 + n2


def is_enabled() -> bool:
    """Example usage of app settings."""
    return app_settings.HTMX_MESSAGES_ENABLED



from django_htmx_messages.main import add, is_enabled


def test_add():
    """Adding two number works as expected."""
    assert add(1, 1) == 2


def test_is_enabled_with_default_value():
    """Enabled function works as expected when unset."""
    assert is_enabled() is True


def test_is_enabled_when_overridden(settings):
    """Enabled function works as expected when setting is changed."""
    settings.HTMX_MESSAGES_ENABLED = False
    assert is_enabled() is False
